/*
 * qsock_addr.cpp
 *
 *  Created on: May 17, 2013
 *      Author: root
 */

#include "../Includes/qstdinc.h"
#include "../Includes/qsock.h"

#include <arpa/inet.h>


QNetAddr::QNetAddr()
{
	Reset();								// Assign later
}

QNetAddr::QNetAddr(const QNetAddr& src)
{
	CopyFrom(src);
}

QNetAddr& QNetAddr::operator=(const QNetAddr& src)
{
	CopyFrom(src);
	return *this;
}

QNetAddr::~QNetAddr()
{
	/* Do nothing */
}

void QNetAddr::CopyFrom(const QNetAddr& src)
{
	m_bValid = src.m_bValid;
	m_nAddr = src.m_nAddr;
	m_nPort = src.m_nPort;
	m_sFullAddr = src.m_sFullAddr;
	m_sIPOnly = src.m_sIPOnly;
}

QNetAddr::QNetAddr(const char* pAddr)
{
	SetAddr(pAddr);
}

QNetAddr::QNetAddr(const char* pAddr, uint16_t nPort)
{
	SetAddr(pAddr, nPort);
}

QNetAddr::QNetAddr(uint32_t nAddr, uint16_t nPort)
{
	SetAddr(nAddr, nPort);
}

#ifndef _WIN32
QNetAddr::QNetAddr(int nSock, bool bPeer)
#else
QNetAddr::QNetAddr(SOCKET nSock, bool bPeer)
#endif
{
	SetAddr(nSock, bPeer);
}

bool QNetAddr::SetAddr(const char* pAddr)
{
	const char* pDL = strchr(pAddr, ':');
	if (pDL == NULL)
	{
		Reset();
	}
	else
	{
		/* Populate to vars */
		m_sIPOnly.assign(pAddr, pDL-pAddr);
		SetAddr(m_sIPOnly.c_str(), (uint16_t)atol(++pDL));
	}
	return m_bValid;
}

bool QNetAddr::SetAddr(const char* pAddr, uint16_t nPort)
{
	SetAddr(inet_addr(pAddr), nPort);
	return m_bValid;
}

bool QNetAddr::SetAddr(uint32_t nAddr, uint16_t nPort)
{
	m_nAddr = nAddr;
	m_nPort = nPort;
	m_bValid = true;
	Expand();
	return m_bValid;
}

void QNetAddr::Expand()
{
	in_addr addr;
	addr.s_addr = m_nAddr;
	char buffer[INET_ADDRSTRLEN + 1];
	const char* pAddr = inet_ntop(AF_INET, (const void*)&addr, buffer, sizeof(buffer));
	if (pAddr)
	{
		m_sIPOnly.assign(buffer);
	}
	else
	{
		m_sIPOnly.assign("<undefined>");
	}
	std::ostringstream stringStream;
	stringStream << m_sIPOnly << ":" << m_nPort;
	m_sFullAddr = stringStream.str();
}

#ifndef _WIN32
bool QNetAddr::SetAddr(int nSock, bool bPeer)
#else
bool QNetAddr::SetAddr(SOCKET nSock, bool bPeer)
#endif
{
	struct sockaddr_in sin;
#ifndef _WIN32
	socklen_t len = sizeof(sin);
#else
	int len = sizeof(sin);
#endif
	if (nSock >= 0)
	{
		int nRetVal = bPeer?getpeername(nSock, (struct sockaddr *)&sin, &len):getsockname(nSock, (struct sockaddr *)&sin, &len);
		if (nRetVal == -1)
		{
			Reset();
		}
		else
		{
			SetAddr(sin.sin_addr.s_addr, htons(sin.sin_port));
		}
	}
	else
	{
		Reset();
	}

	return m_bValid;
}

bool QNetAddr::IsValid()
{
	return m_bValid;
}

uint32_t QNetAddr::GetAddr()
{
	return m_nAddr;
}

uint16_t QNetAddr::GetPort()
{
	return m_nPort;
}

const char* QNetAddr::GetAddrFull()
{
	return m_sFullAddr.c_str();
}

const char* QNetAddr::GetAddrIP()
{
	return m_sIPOnly.c_str();
}

void QNetAddr::Reset()
{
	m_bValid = false;
	m_nAddr = INADDR_ANY;
	m_nPort = 0;
	m_sFullAddr.assign("0.0.0.0:0");
	m_sIPOnly.assign("0.0.0.0");
}

QNetAddr* QNetAddr::Duplicate()
{
	QNetAddr* pRetVal = new QNetAddr();

	CopyTo(pRetVal);

	return pRetVal;
}

void QNetAddr::CopyTo(QNetAddr* pDst)
{
	pDst->m_bValid = m_bValid;
	pDst->m_nAddr = m_nAddr;
	pDst->m_nPort = m_nPort;
	pDst->m_sFullAddr = m_sFullAddr;
	pDst->m_sIPOnly = m_sIPOnly;
}
